@@ -29,7 +29,7 @@
 namespace BeBoB {
 namespace Edirol {
 
-class EdirolFa101Device : public BeBoB::AvDevice {
+class EdirolFa101Device : public BeBoB::Device {
 public:
     EdirolFa101Device( DeviceManager& d,
                        std::auto_ptr<ConfigRom>( configRom ));
