@@ -448,7 +448,7 @@ SaffireProDevice::updateClockSources() {
             m_active_clocksource = &m_wordclock_clocksource;
             break;
     }
-    switch((sync && FR_SAFFIREPRO_CMD_ID_SYNC_LOCK_MASK) >> 8) {
+    switch((sync & FR_SAFFIREPRO_CMD_ID_SYNC_LOCK_MASK) >> 8) {
         case FR_SAFFIREPRO_CMD_SYNC_CONFIG_INTERNAL:
             // always locked
             break;
@@ -545,7 +545,7 @@ uint64_t
 SaffireProDevice::getConfigurationId()
 {
     // have the generic mechanism create a unique configuration id.
-    uint64_t id = BeBoB::AvDevice::getConfigurationId();
+    uint64_t id = BeBoB::Device::getConfigurationId();
 
     // there are some parts that can be enabled/disabled and
     // that have influence on the AV/C model and channel config
@@ -1158,10 +1158,6 @@ SaffireProDeviceStandaloneEnum::count()
 std::string
 SaffireProDeviceStandaloneEnum::getEnumLabel(int idx)
 {
-    if(idx>1) {
-        debugError("Index (%d) out of range\n", idx);
-        return false;
-    }
     switch(idx) {
         case 0: return "Mixing";
         case 1: return "Tracking";
