@@ -23,15 +23,16 @@
  */
 
 #include "debugmodule.h"
-#include "config.h"
 
 #include <stdarg.h>
 #include "libutil/ByteSwap.h"
+#include "libutil/Time.h"
 
 #include <iostream>
 
 #include <time.h>
 #include <string.h>
+#include <errno.h>
 
 #if DEBUG_BACKTRACE_SUPPORT
     #include <execinfo.h>
@@ -42,7 +43,7 @@
 
 #if DEBUG_USE_MESSAGE_BUFFER
 #else
-    #ifdef DEBUG
+    #ifdef DEBUG_MESSAGES
         #warning Printing debug info without ringbuffer, not RT-safe!
     #else
         #error Printing debug info without ringbuffer, not RT-safe (not allowed for non-debug builds)!
@@ -84,7 +85,8 @@ DebugModule::~DebugModule()
 //              << " at DebugModuleManager"
 //              << endl;
 //     }
-    if ( !DebugModuleManager::instance()->unregisterModule( *this ) ) {
+
+    if (m_manager && !m_manager->unregisterModule( *this ) ) {
         cerr << "Could not unregister DebugModule at DebugModuleManager"
              << endl;
     }
@@ -179,7 +181,7 @@ DebugModule::print( debug_level_t level,
 
     // add a timing timestamp
     struct timespec ts;
-    clock_gettime(CLOCK_MONOTONIC, &ts);
+    Util::SystemTimeSource::clockGettime(&ts);
     uint64_t ts_usec=(uint64_t)(ts.tv_sec * 1000000LL + ts.tv_nsec / 1000LL);
 
     // format the message such that it remains together
@@ -263,14 +265,11 @@ DebugModuleManager::DebugModuleManager()
 
 DebugModuleManager::~DebugModuleManager()
 {
-    // cleanin up leftover modules
-    for ( DebugModuleVectorIterator it = m_debugModules.begin();
-          it != m_debugModules.end();
-          ++it )
+    // cleaning up leftover modules
+    while (!m_debugModules.empty())
     {
-        fprintf(stderr,"Cleaning up leftover debug module: %s\n",(*it)->getName().c_str());
-        m_debugModules.erase( it );
-        delete *it;
+        DebugModule *mod = m_debugModules.back();
+        unregisterModule(*mod);
     }
 
     if (!mb_initialized)
@@ -374,22 +373,22 @@ DebugModuleManager::init()
     struct sched_param rt_param;
     pthread_attr_init(&attributes);
     if ((res = pthread_attr_setinheritsched(&attributes, PTHREAD_EXPLICIT_SCHED))) {
-        fprintf(stderr, "Cannot request explicit scheduling for RT thread  %d %s\n", res, strerror(res));
+        fprintf(stderr, "Cannot request explicit scheduling for messagebuffer thread: %s (%d)\n", strerror(res), res);
         return -1;
     }
     if ((res = pthread_attr_setdetachstate(&attributes, PTHREAD_CREATE_JOINABLE))) {
-        fprintf(stderr, "Cannot request joinable thread creation for RT thread  %d %s\n", res, strerror(res));
+        fprintf(stderr, "Cannot request joinable thread creation for messagebuffer thread: %s (%d)\n", strerror(res), res);
         return -1;
     }
     if ((res = pthread_attr_setscope(&attributes, PTHREAD_SCOPE_SYSTEM))) {
-        fprintf(stderr, "Cannot set scheduling scope for RT thread %d %s\n", res, strerror(res));
+        fprintf(stderr, "Cannot set scheduling scope for messagebuffer thread: %s (%d)\n", strerror(res), res);
         return -1;
     }
 
     if ((res = pthread_attr_setschedpolicy(&attributes, SCHED_FIFO))) {
 
     //if ((res = pthread_attr_setschedpolicy(&attributes, SCHED_RR))) {
-        fprintf(stderr, "Cannot set FIFO scheduling class for RT thread  %d %s\n", res, strerror(res));
+        fprintf(stderr, "Cannot set FIFO scheduling class for messagebuffer thread: %s (%d)\n", strerror(res), res);
         return -1;
     }
 
@@ -397,15 +396,31 @@ DebugModuleManager::init()
     rt_param.sched_priority = DEBUG_MESSAGE_BUFFER_REALTIME_PRIO;
 
     if ((res = pthread_attr_setschedparam(&attributes, &rt_param))) {
-        fprintf(stderr, "Cannot set scheduling priority for RT thread %d %s\n", res, strerror(res));
+        fprintf(stderr, "Cannot set scheduling priority for messagebuffer thread: %s (%d)\n", strerror(res), res);
         return -1;
     }
 
     mb_initialized = 1; // set to 1 otherwise the thread might exit (race condition)
     if ((res = pthread_create(&mb_writer_thread, &attributes, mb_thread_func, (void *)this))) {
-        fprintf(stderr, "Cannot create thread %d %s\n", res, strerror(res));
+        fprintf(stderr, "Cannot create RT messagebuffer thread: %s (%d)\n", strerror(res), res);
         mb_initialized = 0;
     }
+
+    if (res==EPERM && mb_initialized==0) {
+        fprintf(stderr, "Retrying messagebuffer thread without RT scheduling\n");
+        memset(&rt_param, 0, sizeof(rt_param));
+        if ((res=pthread_attr_setschedpolicy(&attributes, SCHED_OTHER)) ||
+            (res=pthread_attr_setschedparam(&attributes, &rt_param))) {
+            fprintf(stderr, "Cannot set standard scheduling for messagebuffer thread: %s (%d)\n", strerror(res), res);
+        } else {
+            mb_initialized = 1; // set to 1 otherwise the thread might exit (race condition)
+            if ((res = pthread_create(&mb_writer_thread, &attributes, mb_thread_func, (void *)this))) {
+                fprintf(stderr, "Cannot create messagebuffer thread: %s (%d)\n", strerror(res), res);
+                mb_initialized = 0;
+            } else
+                fprintf(stderr, "Messagebuffer not realtime; consider enabling RT scheduling for user\n");
+        }
+    }
     #else
     mb_initialized = 1; // set to 1 otherwise the thread might exit (race condition)
     if ((res = pthread_create(&mb_writer_thread, NULL, &mb_thread_func, (void *)this))) {
@@ -455,6 +470,8 @@ DebugModuleManager::registerModule( DebugModule& debugModule )
             << "DebugModule (" << debugModule.getName() << ")" << endl;
     } else {
         m_debugModules.push_back( &debugModule );
+        if (debugModule.m_manager == NULL)
+                debugModule.m_manager = this;
     }
     return true;
 }
@@ -469,6 +486,8 @@ DebugModuleManager::unregisterModule( DebugModule& debugModule )
     {
         if ( *it == &debugModule ) {
             m_debugModules.erase( it );
+            if (debugModule.m_manager == this)
+                debugModule.m_manager = NULL;
             return true;
         }
     }
@@ -673,7 +692,7 @@ DebugModuleManager::printBacktrace(int len)
         char name[64];
         name[0]=0;
         getFunctionName(m_backtrace_buffer[j], name, 64);
-        chars_written += snprintf(m_backtrace_strbuffer + chars_written, MB_BUFFERSIZE-chars_written, "%s ", name);
+        chars_written += snprintf(m_backtrace_strbuffer + chars_written, MB_BUFFERSIZE-chars_written, "%s\n", name);
     }
     chars_written += snprintf(m_backtrace_strbuffer + chars_written, MB_BUFFERSIZE-chars_written, "\n");
 
@@ -860,7 +879,7 @@ hexDumpQuadlets( quadlet_t *data, unsigned int length )
         return;
     }
     for (i = 0; i< length; i += 1) {
-        printf( "%02d %04X: %08X (%08X)"
+        fprintf(stderr, "%02d %04X: %08X (%08X)"
                 "\n", i, i*4, data[i],CondSwapFromBus32(data[i]));
     }
 }
