@@ -25,7 +25,7 @@
 #ifndef DEBUGMODULE_H
 #define DEBUGMODULE_H
 
-#include "config.h"
+#include "config_debug.h"
 
 #include "../fbtypes.h"
 #include <assert.h>
@@ -36,6 +36,18 @@
 #include <stdint.h>
 #include <semaphore.h>
 
+#define FFADO_ASSERT(x) { \
+    if(!(x)) { \
+        m_debugModule.print( DebugModule::eDL_Fatal,        \
+                                __FILE__,                   \
+                                __FUNCTION__,               \
+                                __LINE__,                   \
+                                "Assertion failed...\n");      \
+        debugPrintBacktrace( 10 ); \
+        DebugModuleManager::instance()->flush(); \
+        assert(x); \
+    }}
+
 typedef short debug_level_t;
 
 #define DEBUG_LEVEL_MESSAGE        0
@@ -104,8 +116,8 @@ typedef short debug_level_t;
                                      ##args )
 
 // these are for messages that are also displayed when not compiled
-// for debug
-#ifdef DEBUG
+// for debug messages
+#ifdef DEBUG_MESSAGES
 #define printMessage( format, args... )                             \
                 m_debugModule.print( DebugModule::eDL_Message,      \
                                      __FILE__,                      \
@@ -179,7 +191,7 @@ typedef short debug_level_t;
 
 #endif
 
-#ifdef DEBUG
+#ifdef DEBUG_MESSAGES
 
     #define debugOutput( level, format, args... )                  \
                 m_debugModule.print( level,                        \
@@ -270,8 +282,12 @@ void quadlet2char( fb_quadlet_t quadlet, unsigned char* buff );
 void hexDump( unsigned char *data_start, unsigned int length );
 void hexDumpQuadlets( quadlet_t *data_start, unsigned int length );
 
+class DebugModuleManager;
+
 class DebugModule {
 public:
+    friend class DebugModuleManager;
+
     enum {
         eDL_Message      = DEBUG_LEVEL_MESSAGE,
         eDL_Fatal        = DEBUG_LEVEL_FATAL,
@@ -320,6 +336,7 @@ protected:
 private:
     std::string   m_name;
     debug_level_t m_level;
+    DebugModuleManager* m_manager;
 };
 
 
