@@ -67,15 +67,15 @@ public:
      * @brief Compares the GUID of two FFADODevices
      *
      * This function compares the GUID of two FFADODevices and returns true
-     * if the GUID of @ref a is larger than the GUID of @ref b . This is intended
-     * to be used with the STL sort() algorithm.
+     * if the GUID of a is larger than the GUID of b . This is intended to be
+     * used with the STL sort() algorithm.
      * 
      * Note that GUID's are converted to integers for this.
      * 
      * @param a pointer to first FFADODevice
      * @param b pointer to second FFADODevice
      * 
-     * @returns true if the GUID of @ref a is larger than the GUID of @ref b .
+     * @returns true if the GUID of a is larger than the GUID of b .
      */
     static bool compareGUID( FFADODevice *a, FFADODevice *b );
 
@@ -148,6 +148,19 @@ public:
      */
     virtual int getSamplingFrequency( ) = 0;
 
+    /**
+     * @brief Some devices require to update part of their internal configuration on samplerate change
+     *        Most do not, so set default to false
+     * @param oldSamplingFrequency : to be compared to the actual sampling frequency of the device
+     *        note: next, it is thus assumed that the device was actually set to some new sampling frequency
+     *        (while the client did not yet update its view of the device).
+     *        In some future, it might very well appear that restricting to this single argument is
+     *        unsufficient for some devices, so feel free to adapt accordingly.
+     *
+     * @return true if changes have been performed
+     */
+    virtual bool onSamplerateChange( int oldSamplingFrequency ) { return false; }
+
     /**
      * @brief get the supported sampling frequencies
      * @return a vector containing the supported sampling frequencies
@@ -275,36 +288,6 @@ public:
      */
     virtual enum eStreamingState getStreamingState();
 
-    /**
-     * @brief This is called by the device manager to give the device a unique ID.
-     *
-     * The purpose of this is to allow for unique port naming
-     * in case there are multiple identical devices on the bus.
-     * Some audio API's (e.g. jack) don't work properly when the
-     * port names are not unique.
-     *
-     * Say you have two devices having a port named OutputLeft.
-     * This can cause the streaming
-     * part to present two OutputLeft ports to the audio API,
-     * which won't work. This ID will allow you to construct
-     * the port names as 'dev1_OutputLeft' and 'dev2_OutputLeft'
-     *
-     * @note Currently this is a simple integer that is equal to
-     *       the position of the device in the devicemanager's
-     *       device list. Therefore it is dependant on the order
-     *       in which the devices are detected. The side-effect
-     *       of this is that it is dependant on the bus topology
-     *       and history (e.g. busresets etc). This makes that
-     *       these ID's are not fixed to a specific physical device.
-     *       At some point, we will replaced this with a GUID based
-     *       approach, which is tied to a physical device and is
-     *       bus & time independant.
-     *
-     * @param id
-     * @return true if successful
-     */
-    bool setId(unsigned int id);
-
     /**
      * @brief Outputs the device configuration to stderr/stdout [debug helper]
      *
@@ -377,6 +360,18 @@ public:
      */
     virtual bool prepare() = 0;
 
+    /**
+     * @brief Performs operations needed to prepare for a stream start
+     *
+     * This is called by the streaming layer just before streaming is
+     * started.  It provides a place where reset activity can be done which
+     * ensures the object is ready to restart streaming even if streaming
+     * has been previously started and stopped.
+     *
+     * @return true if successful, false if not
+     */
+    virtual bool resetForStreaming() { return true; }
+
     /**
      * @brief Returns the number of ISO streams implemented/used by this device
      *
@@ -404,8 +399,7 @@ public:
      * @note you cannot have two streamprocessors handling the same ISO
      *       channel (on the same port)
      *
-     * @param i : Stream index
-     * @pre @ref i smaller than getStreamCount()
+     * @param i : Stream index (i has to be smaller than getStreamCount())
      * @return a StreamProcessor object if successful, NULL otherwise
      */
     virtual Streaming::StreamProcessor *getStreamProcessorByIndex(int i) = 0;
@@ -421,17 +415,15 @@ public:
      * Channel allocation should be done using the allocation functions provided by the
      * Ieee1394Service object that is passed in the constructor.
      *
-     * @param i : Stream index
-     * @pre @ref i smaller than getStreamCount()
+     * @param i : Stream index (i has to be smaller than getStreamCount())
      * @return true if successful, false if not
      */
     virtual bool startStreamByIndex(int i) = 0;
 
     /**
-     * @brief stops the stream with index @ref i
+     * @brief stops the stream with index i
      *
-     * @param i : Stream index
-     * @pre @ref i smaller than getStreamCount()
+     * @param i : Stream index (i has to be smaller than getStreamCount())
      * @return true if successful, false if not
      */
     virtual bool stopStreamByIndex(int i) = 0;
