@@ -21,12 +21,10 @@
  *
  */
 
+// #include "config.h"
 #include "devicemanager.h"
 #include "fireworks_device.h"
 #include "efc/efc_avc_cmd.h"
-#include "efc/efc_cmd.h"
-#include "efc/efc_cmds_hardware.h"
-#include "efc/efc_cmds_hardware_ctrl.h"
 #include "efc/efc_cmds_flash.h"
 
 #include "audiofire/audiofire_device.h"
@@ -44,13 +42,14 @@
 #define FIREWORKS_MIN_FIRMWARE_VERSION 0x04080000
 
 #include <sstream>
+#include <unistd.h>
 using namespace std;
 
 // FireWorks is the platform used and developed by ECHO AUDIO
 namespace FireWorks {
 
 Device::Device(DeviceManager& d, std::auto_ptr<ConfigRom>( configRom ))
-    : GenericAVC::AvDevice( d, configRom)
+    : GenericAVC::Device( d, configRom)
     , m_poll_lock( new Util::PosixMutex("DEVPOLL") )
     , m_efc_discovery_done ( false )
     , m_MixerContainer ( NULL )
@@ -75,7 +74,7 @@ Device::showDevice()
         }
     }
     m_HwInfo.showEfcCmd();
-    GenericAVC::AvDevice::showDevice();
+    GenericAVC::Device::showDevice();
 }
 
 bool
@@ -110,7 +109,6 @@ Device::probe( Util::Configuration& c, ConfigRom& configRom, bool generic )
     } else {
         unsigned int vendorId = configRom.getNodeVendorId();
         unsigned int modelId = configRom.getModelId();
-
         Util::Configuration::VendorModelEntry vme = c.findDeviceVME( vendorId, modelId );
         return c.isValid(vme) && vme.driver == Util::Configuration::eD_FireWorks;
     }
@@ -140,8 +138,8 @@ Device::discover()
     }
 
     // discover AVC-wise
-    if ( !GenericAVC::AvDevice::discoverGeneric() ) {
-        debugError( "Could not discover GenericAVC::AvDevice\n" );
+    if ( !GenericAVC::Device::discoverGeneric() ) {
+        debugError( "Could not discover GenericAVC::Device\n" );
         return false;
     }
 
@@ -176,14 +174,7 @@ Device::discoverUsingEFC()
         return false;
     }
 
-    // save the EFC version, since some stuff
-    // depends on this
-    m_efc_version = m_HwInfo.m_header.version;
-
-    if (!updatePolledValues()) {
-        debugError("Could not update polled values\n");
-        return false;
-    }
+    m_current_clock = -1;
 
     m_efc_discovery_done = true;
     return true;
@@ -201,8 +192,8 @@ Device::createDevice(DeviceManager& d, std::auto_ptr<ConfigRom>( configRom ))
     }
 }
 
-bool 
-Device::doEfcOverAVC(EfcCmd &c) {
+bool Device::doEfcOverAVC(EfcCmd &c)
+{
     EfcOverAVCCmd cmd( get1394Service() );
     cmd.setCommandType( AVC::AVCCommand::eCT_Control );
     cmd.setNodeId( getConfigRom().getNodeId() );
@@ -212,7 +203,7 @@ Device::doEfcOverAVC(EfcCmd &c) {
     cmd.setVerbose( getDebugLevel() );
     cmd.m_cmd = &c;
 
-    if ( !cmd.fire()) {
+    if (!cmd.fire()) {
         debugError( "EfcOverAVCCmd command failed\n" );
         c.showEfcCmd();
         return false;
@@ -317,6 +308,10 @@ Device::buildMixer()
         new HwInfoControl(*this, HwInfoControl::eHIF_GroupInCount, "GroupInCount"));
     result &= m_HwInfoContainer->addElement(
         new HwInfoControl(*this, HwInfoControl::eHIF_PhantomPower, "PhantomPower"));
+    result &= m_HwInfoContainer->addElement(
+        new HwInfoControl(*this, HwInfoControl::eHIF_OpticalInterface, "OpticalInterface"));
+    result &= m_HwInfoContainer->addElement(
+        new HwInfoControl(*this, HwInfoControl::eHIF_PlaybackRouting, "PlaybackRouting"));
 
     // add a save settings control
     result &= this->addElement(
@@ -335,10 +330,18 @@ Device::buildMixer()
         result &= this->addElement(
             new IOConfigControl(*this, eCR_Mirror, "ChannelMirror"));
     }
+    if(m_HwInfo.hasOpticalInterface()) {
+        result &= this->addElement(
+            new IOConfigControl(*this, eCR_DigitalInterface, "DigitalInterface"));
+    }
     if(m_HwInfo.hasSoftwarePhantom()) {
         result &= this->addElement(
             new IOConfigControl(*this, eCR_Phantom, "PhantomPower"));
     }
+    if(m_HwInfo.hasPlaybackRouting()) {
+        result &= this->addElement(
+            new PlaybackRoutingControl(*this, "PlaybackRouting"));
+    }
 
     if (!result) {
         debugWarning("One or more control elements could not be created.");
@@ -429,6 +432,11 @@ Device::loadSession()
     return true;
 }
 
+/*
+ * NOTE:
+ * Firmware version 5.0 or later for AudioFire12 returns invalid values to
+ * contents of response against this command.
+ */
 bool
 Device::updatePolledValues() {
     Util::MutexLockHelper lock(*m_poll_lock);
@@ -463,7 +471,7 @@ Device::getSupportedClockSources() {
         return r;
     }
 
-    uint32_t active_clock=getClock();
+    uint32_t active_clock = getClockSrc();
 
     if(EFC_CMD_HW_CHECK_FLAG(m_HwInfo.m_supported_clocks, EFC_CMD_HW_CLOCK_INTERNAL)) {
         debugOutput(DEBUG_LEVEL_VERBOSE, "Internal clock supported\n");
@@ -507,10 +515,10 @@ Device::getSupportedClockSources() {
 bool
 Device::isClockValid(uint32_t id) {
     // always valid
-    if (id==EFC_CMD_HW_CLOCK_INTERNAL) return true;
+    if (id==EFC_CMD_HW_CLOCK_INTERNAL)
+        return true;
 
-    // the polled values are used to detect
-    // whether clocks are valid
+    // the polled values tell whether each clock source is detected or not
     if (!updatePolledValues()) {
         debugError("Could not update polled values\n");
         return false;
@@ -529,7 +537,7 @@ Device::setActiveClockSource(ClockSource s) {
         return false;
     }
 
-    result=setClock(s.id);
+    result = setClockSrc(s.id);
 
     // From the ECHO sources:
     // "If this is a 1200F and the sample rate is being set via EFC, then
@@ -550,7 +558,7 @@ Device::setActiveClockSource(ClockSource s) {
 FFADODevice::ClockSource
 Device::getActiveClockSource() {
     ClockSource s;
-    uint32_t active_clock=getClock();
+    uint32_t active_clock = getClockSrc();
     s=clockIdToClockSource(active_clock);
     s.active=true;
     return s;
@@ -561,13 +569,6 @@ Device::clockIdToClockSource(uint32_t clockid) {
     ClockSource s;
     debugOutput(DEBUG_LEVEL_VERBOSE, "clock id: %u\n", clockid);
 
-    // the polled values are used to detect
-    // whether clocks are valid
-    if (!updatePolledValues()) {
-        debugError("Could not update polled values\n");
-        return s;
-    }
-
     switch (clockid) {
         case EFC_CMD_HW_CLOCK_INTERNAL:
             debugOutput(DEBUG_LEVEL_VERBOSE, "Internal clock\n");
@@ -616,38 +617,134 @@ Device::clockIdToClockSource(uint32_t clockid) {
     return s;
 }
 
-uint32_t
-Device::getClock() {
-    EfcGetClockCmd gccmd;
-    if (!doEfcOverAVC(gccmd)) {
-        debugError("Could not get clock info\n");
-        return EFC_CMD_HW_CLOCK_UNSPECIFIED;
+bool Device::getClock(EfcGetClockCmd &gccmd)
+{
+    if (!doEfcOverAVC(gccmd))
+        return false;
+
+    /*
+     * NOTE:
+     * Firmware version 5.0 or later for AudioFire12 returns invalid
+     * values in contents of response against this command.
+     */
+    if (gccmd.m_samplerate > 192000) {
+        debugOutput(DEBUG_LEVEL_NORMAL,
+                    "Could not get sampling rate. Do fallback\n");
+        int sampling_rate;
+
+        /* fallback to 'input/output plug signal format' command */
+        sampling_rate = GenericAVC::Device::getSamplingFrequency();
+        /* fallback failed */
+        if (!sampling_rate) {
+            debugOutput(DEBUG_LEVEL_NORMAL, "Fallback failed\n");
+            return false;
+        }
+
+        gccmd.m_samplerate = sampling_rate;
     }
-    debugOutput(DEBUG_LEVEL_VERBOSE, "Active clock: 0x%08X\n",gccmd.m_clock);
-    gccmd.showEfcCmd();
+
+    if (gccmd.m_clock > EFC_CMD_HW_CLOCK_COUNT) {
+        debugOutput(DEBUG_LEVEL_NORMAL,
+                    "Could not get clock info. Do fallback\n");
+        if (m_current_clock < 0) {
+            /* fallback to internal clock source */
+            EfcSetClockCmd sccmd;
+            sccmd.m_clock = EFC_CMD_HW_CLOCK_INTERNAL;
+            sccmd.m_samplerate = gccmd.m_samplerate;
+            sccmd.m_index = 0;
+
+            if (!doEfcOverAVC(sccmd)) {
+                debugOutput(DEBUG_LEVEL_NORMAL, "Fallback failed\n");
+                return false;
+            }
+
+            /* Cache clock source */
+            m_current_clock = sccmd.m_clock;
+        }
+
+        /* Fallback to cache */
+        gccmd.m_clock = m_current_clock;
+    }
+
+    return true;
+}
+uint32_t Device::getClockSrc()
+{
+    EfcGetClockCmd gccmd;
+    if (!getClock(gccmd))
+        return EFC_CMD_HW_CLOCK_UNSPECIFIED;
+
+    debugOutput(DEBUG_LEVEL_VERBOSE, "Get current clock source: %d\n",
+                gccmd.m_clock);
 
     return gccmd.m_clock;
 }
-
-bool
-Device::setClock(uint32_t id) {
+int Device::getSamplingFrequency()
+{
     EfcGetClockCmd gccmd;
-    if (!doEfcOverAVC(gccmd)) {
-        debugError("Could not get clock info\n");
-        return false;
-    }
-    debugOutput(DEBUG_LEVEL_VERBOSE, "Set clock: 0x%08X\n", id);
+    if (!getClock(gccmd))
+        return 0;
 
-    EfcSetClockCmd sccmd;
-    sccmd.m_clock=id;
-    sccmd.m_samplerate=gccmd.m_samplerate;
-    sccmd.m_index=0;
+    debugOutput(DEBUG_LEVEL_VERBOSE, "Get current sample rate: %d\n",
+                gccmd.m_samplerate);
+
+    return gccmd.m_samplerate;
+}
+
+bool Device::setClock(EfcSetClockCmd sccmd)
+{
     if (!doEfcOverAVC(sccmd)) {
         debugError("Could not set clock info\n");
         return false;
     }
+
+    /* Cache clock source for fallback. */
+    m_current_clock = sccmd.m_clock;
+
     return true;
 }
+bool Device::setClockSrc(uint32_t id)
+{
+    bool err;
+
+    EfcGetClockCmd gccmd;
+    err = getClock(gccmd);
+    if (!err)
+        return err;
+
+    EfcSetClockCmd sccmd;
+    sccmd.m_clock = id;
+    sccmd.m_samplerate = gccmd.m_samplerate;
+    sccmd.m_index = 0;
+
+    err = setClock(sccmd);
+    if (err)
+        debugOutput(DEBUG_LEVEL_VERBOSE, "Set current clock source: %d\n",
+                    sccmd.m_clock);
+
+    return err;
+}
+bool Device::setSamplingFrequency(int samplerate)
+{
+    bool err;
+
+    EfcGetClockCmd gccmd;
+    err = getClock(gccmd);
+    if (!err)
+        return err;
+
+    EfcSetClockCmd sccmd;
+    sccmd.m_clock = gccmd.m_clock;
+    sccmd.m_samplerate = samplerate;
+    sccmd.m_index = 0;
+
+    err = setClock(sccmd);
+    if (err)
+        debugOutput(DEBUG_LEVEL_VERBOSE, "Set current sample rate: %d\n",
+                    sccmd.m_samplerate);
+
+    return err;
+}
 
 bool
 Device::lockFlash(bool lock) {
@@ -889,4 +986,5 @@ Device::getSessionBase()
     return cmd.m_address;
 }
 
+
 } // FireWorks
