@@ -127,7 +127,7 @@ Plug::Plug( const Plug& rhs )
     , m_name( rhs.m_name )
     , m_clusterInfos( rhs.m_clusterInfos )
     , m_formatInfos( rhs.m_formatInfos )
-// FIXME: how about the global id?
+    , m_globalId( rhs.m_globalId )
 {
     if ( getDebugLevel() ) {
         setDebugLevel( DEBUG_LEVEL_VERBOSE );
@@ -236,7 +236,11 @@ Plug::discover()
 //         return false;
     }
 
-    if ( !discoverSupportedStreamFormats() ) {
+    // adi@2011-1-14: short-circuit left-to-right evaluation, so only try to
+    // discover the supported stream formats when not on a sync channel.
+    // According to Holger Dehnhardt, this is required to make his Mackie
+    // Onyx work.
+    if ( (m_infoPlugType != AVC::Plug::eAPT_Sync) && !discoverSupportedStreamFormats() ) {
         debugOutput(DEBUG_LEVEL_NORMAL, 
                     "Could not discover supported stream formats "
                     "(%d,%d,%d,%d,%d)\n",
@@ -288,8 +292,65 @@ Plug::discoverName()
 bool
 Plug::discoverNoOfChannels()
 {
-
-    return true;
+    if (m_nrOfChannels == 0) {
+        // not discovered yet, get from ext stream format
+        ExtendedStreamFormatCmd extStreamFormatCmd =
+            setPlugAddrToStreamFormatCmd( ExtendedStreamFormatCmd::eSF_ExtendedStreamFormatInformationCommand );
+        extStreamFormatCmd.setVerbose( getDebugLevel() );
+    
+        if ( !extStreamFormatCmd.fire() ) {
+            debugError( "stream format command failed\n" );
+            return false;
+        }
+    
+        if ( ( extStreamFormatCmd.getStatus() ==  ExtendedStreamFormatCmd::eS_NoStreamFormat )
+            || ( extStreamFormatCmd.getStatus() ==  ExtendedStreamFormatCmd::eS_NotUsed ) )
+        {
+            debugOutput( DEBUG_LEVEL_VERBOSE,
+                        "No stream format information available\n" );
+            return true;
+        }
+    
+        if ( !extStreamFormatCmd.getFormatInformation() ) {
+            debugOutput(DEBUG_LEVEL_NORMAL, "No stream format information for plug found -> skip\n" );
+            return true;
+        }
+    
+        if ( extStreamFormatCmd.getFormatInformation()->m_root
+            != FormatInformation::eFHR_AudioMusic  )
+        {
+            debugOutput(DEBUG_LEVEL_NORMAL, "Format hierarchy root is not Audio&Music -> skip\n" );
+            return true;
+        }
+    
+        FormatInformation* formatInfo =
+            extStreamFormatCmd.getFormatInformation();
+        FormatInformationStreamsCompound* compoundStream
+            = dynamic_cast< FormatInformationStreamsCompound* > (
+                formatInfo->m_streams );
+        if ( compoundStream ) {
+            unsigned int nb_channels = 0;
+            for ( int i = 1;
+                i <= compoundStream->m_numberOfStreamFormatInfos;
+                ++i )
+            {
+                StreamFormatInfo* streamFormatInfo =
+                    compoundStream->m_streamFormatInfos[ i - 1 ];
+    
+                debugOutput( DEBUG_LEVEL_VERBOSE,
+                            "number of channels = %d, stream format = %d\n",
+                            streamFormatInfo->m_numberOfChannels,
+                            streamFormatInfo->m_streamFormat );
+                // FIXME: might not be correct to sum all
+                nb_channels += streamFormatInfo->m_numberOfChannels;
+            }
+            m_nrOfChannels = nb_channels;
+        }
+        return true;
+    } else {
+        // already got the nb channels from somewhere else
+        return true;
+    }
 }
 
 bool
@@ -309,7 +370,29 @@ Plug::discoverChannelName()
 bool
 Plug::discoverClusterInfo()
 {
+    // if there are no cluster info's, we'll have to come up with some
+    if(m_clusterInfos.size() == 0) {
+        debugOutput( DEBUG_LEVEL_VERBOSE, "fixing up cluster infos\n");
+        // we figure out how many channels we have, and build one cluster
+        struct ClusterInfo c;
+        c.m_index = 1;
+        c.m_portType = 0;
+        c.m_name = "Unknown";
+        c.m_buildSource = -1; // Flag this as a temp build
 
+        c.m_nrOfChannels = m_nrOfChannels;
+        for(int i=0; i<m_nrOfChannels; i++) {
+            struct ChannelInfo ci;
+            ci.m_streamPosition = i;
+            ci.m_location = 0xFF;
+            ci.m_name = "Unknown";
+            c.m_channelInfos.push_back(ci);
+        }
+        c.m_streamFormat = -1; // filled in later
+        
+        m_clusterInfos.push_back(c);
+    }
+    
     return true;
 }
 
@@ -372,7 +455,7 @@ Plug::discoverStreamFormat()
                 debugOutput(DEBUG_LEVEL_NORMAL, 
                             "No matching cluster "
                             "info found for index %d\n",  i );
-                    return false;
+                    //return false;
             }
             StreamFormatInfo* streamFormatInfo =
                 compoundStream->m_streamFormatInfos[ i - 1 ];
@@ -382,36 +465,38 @@ Plug::discoverStreamFormat()
                          streamFormatInfo->m_numberOfChannels,
                          streamFormatInfo->m_streamFormat );
 
-            int nrOfChannels = clusterInfo->m_nrOfChannels;
-            if ( streamFormatInfo->m_streamFormat ==
-                 FormatInformation::eFHL2_AM824_MIDI_CONFORMANT )
-            {
-                // 8 logical midi channels fit into 1 channel
-                nrOfChannels = ( ( nrOfChannels + 7 ) / 8 );
+            if ( clusterInfo ) {
+                int nrOfChannels = clusterInfo->m_nrOfChannels;
+                if ( streamFormatInfo->m_streamFormat ==
+                    FormatInformation::eFHL2_AM824_MIDI_CONFORMANT )
+                {
+                    // 8 logical midi channels fit into 1 channel
+                    nrOfChannels = ( ( nrOfChannels + 7 ) / 8 );
+                }
+                // sanity check
+                if ( nrOfChannels != streamFormatInfo->m_numberOfChannels )
+                {
+                    debugOutput(DEBUG_LEVEL_NORMAL, 
+                                "Number of channels "
+                                "mismatch: '%s' plug discovering reported "
+                                "%d channels for cluster '%s', while stream "
+                                "format reported %d\n",
+                                getName(),
+                                nrOfChannels,
+                                clusterInfo->m_name.c_str(),
+                                streamFormatInfo->m_numberOfChannels);
+                }
+                clusterInfo->m_streamFormat = streamFormatInfo->m_streamFormat;
+    
+                debugOutput( DEBUG_LEVEL_VERBOSE,
+                            "%s plug %d cluster info %d ('%s'): "
+                            "stream format %d\n",
+                            getName(),
+                            m_id,
+                            i,
+                            clusterInfo->m_name.c_str(),
+                            clusterInfo->m_streamFormat );
             }
-            // sanity check
-            if ( nrOfChannels != streamFormatInfo->m_numberOfChannels )
-            {
-                debugOutput(DEBUG_LEVEL_NORMAL, 
-                              "Number of channels "
-                              "mismatch: '%s' plug discovering reported "
-                              "%d channels for cluster '%s', while stream "
-                              "format reported %d\n",
-                              getName(),
-                              nrOfChannels,
-                              clusterInfo->m_name.c_str(),
-                              streamFormatInfo->m_numberOfChannels);
-            }
-            clusterInfo->m_streamFormat = streamFormatInfo->m_streamFormat;
-
-            debugOutput( DEBUG_LEVEL_VERBOSE,
-                         "%s plug %d cluster info %d ('%s'): "
-                         "stream format %d\n",
-                         getName(),
-                         m_id,
-                         i,
-                         clusterInfo->m_name.c_str(),
-                         clusterInfo->m_streamFormat );
         }
     }
 
@@ -482,7 +567,7 @@ Plug::discoverSupportedStreamFormats()
                       ++j )
                 {
                     switch ( compoundStream->m_streamFormatInfos[j]->m_streamFormat ) {
-                    case AVC1394_STREAM_FORMAT_AM824_IEC60968_3:
+                    case AVC1394_STREAM_FORMAT_AM824_IEC60958_3:
                         formatInfo.m_audioChannels +=
                             compoundStream->m_streamFormatInfos[j]->m_numberOfChannels;
                         break;
@@ -754,8 +839,12 @@ Plug::propagateFromPlug( Plug *p ) {
                  "Propagating info from plug '%s' to plug '%s'\n",
                  p->getName(), getName() );
 
-    if (m_clusterInfos.size()==0) {
+    if (m_clusterInfos.size()==0 || m_clusterInfos[0].m_buildSource == -1) {
         m_clusterInfos=p->m_clusterInfos;
+
+    	if (m_clusterInfos.size() > 0) {
+            m_clusterInfos[0].m_buildSource = 0; // No longer a temp instance
+	}
     }
 
     m_nrOfChannels=p->m_nrOfChannels;
@@ -1251,7 +1340,8 @@ Plug::debugOutputClusterInfos( int debugLevel )
               ++cit )
         {
             const ChannelInfo* channelInfo = &( *cit );
-            channelInfo = channelInfo;
+            // Avoid unused variable warning when debug is disabled
+            static_cast<void>(channelInfo);
             debugOutput( debugLevel,
                          "stream position: %d\n",
                          channelInfo->m_streamPosition );
@@ -1342,9 +1432,10 @@ Plug::plugDirectionToString(enum EPlugDirection t) {
             return string("Input");
         case eAPD_Output:
             return string("Output");
-        default:
+        case eAPD_Unknown:
             return string("Unknown");
     }
+    return string("ERROR");
 }
 
 void
