@@ -28,6 +28,7 @@
 #include "fbtypes.h"
 #include "libutil/Functors.h"
 #include "libutil/Mutex.h"
+#include "libutil/Thread.h"
 
 #include "debugmodule/debugmodule.h"
 
@@ -40,6 +41,7 @@
 #include <string>
 #include <stdint.h>
 
+
 #define MAX_FCP_BLOCK_SIZE_BYTES (512)
 #define MAX_FCP_BLOCK_SIZE_QUADS (MAX_FCP_BLOCK_SIZE_BYTES / 4)
 
@@ -52,6 +54,9 @@ namespace Util {
 }
 
 class Ieee1394Service : public IEC61883 {
+public:
+    class ARMHandler;
+
 public:
     Ieee1394Service();
     Ieee1394Service(bool rt, int prio);
@@ -267,6 +272,24 @@ public:
     void doBusReset();
     bool waitForBusResetStormToEnd( int nb_tries, int sleep_time_ms );
 
+    /**
+     * @brief register an AddressRangeMapping Handler
+     * @param h pointer to the handler to register
+     *
+     * @return true on success or false on failure
+     **/
+
+    bool registerARMHandler( ARMHandler *h );
+
+    /**
+     * @brief unregister ARM range
+     * @param h pointer to the handler to unregister
+     * @return true if successful, false otherwise
+     */
+    bool unregisterARMHandler( ARMHandler *h );
+
+    nodeaddr_t findFreeARMBlock( nodeaddr_t start, size_t length, size_t step );
+
     /**
      * @brief get the current generation
      *
@@ -336,6 +359,8 @@ public:
 public:
     signed int getAvailableBandwidth();
     signed int allocateIsoChannelGeneric(unsigned int bandwidth);
+    signed int allocateFixedIsoChannelGeneric(
+        unsigned int chan, unsigned int bandwidth);
     signed int allocateIsoChannelCMP(nodeid_t xmit_node, int xmit_plug,
                                      nodeid_t recv_node, int recv_plug);
     bool freeIsoChannel(signed int channel);
@@ -372,11 +397,42 @@ protected:
     Util::Configuration     *m_configuration;
 
 private:
-    bool configurationUpdated();
+    // this class will create a new 1394 handle
+    // and a thread that will iterate it
+    class HelperThread : public Util::RunnableInterface
+    {
+    public:
+        HelperThread(Ieee1394Service &, std::string);
+        HelperThread(Ieee1394Service &, std::string, bool rt, int prio);
+        virtual ~HelperThread();
 
-    bool startRHThread();
-    void stopRHThread();
-    static void* rHThread( void* arg );
+        raw1394handle_t get1394Handle() {return m_handle;};
+        Ieee1394Service &get1394Service() {return m_parent;};
+
+        virtual bool Init();
+        virtual bool Execute();
+
+        void setThreadParameters(bool rt, int priority);
+
+        bool Start();
+        bool Stop();
+
+    private:
+        Ieee1394Service &m_parent;
+        std::string      m_name;
+        raw1394handle_t  m_handle;
+        Util::Thread &   m_thread;
+        bool             m_iterate;
+
+        DECLARE_DEBUG_MODULE_REFERENCE;
+    };
+
+    HelperThread *m_resetHelper;
+    HelperThread *m_armHelperNormal;
+    HelperThread *m_armHelperRealtime;
+
+private: // unsorted
+    bool configurationUpdated();
 
     void printBuffer( unsigned int level, size_t length, fb_quadlet_t* buffer ) const;
     void printBufferBytes( unsigned int level, size_t length, byte_t* buffer ) const;
@@ -385,23 +441,26 @@ private:
                                      unsigned int generation );
     bool resetHandler( unsigned int generation );
 
+    static int armHandlerLowLevel(raw1394handle_t handle, unsigned long arm_tag,
+                     byte_t request_type, unsigned int requested_length,
+                     void *data);
+    bool armHandler( unsigned long arm_tag,
+                     byte_t request_type, unsigned int requested_length,
+                     void *data);
+
     raw1394handle_t m_handle;
     Util::Mutex*    m_handle_lock;
-    raw1394handle_t m_resetHandle;
-    raw1394handle_t m_util_handle; // a handle for operations from the rt thread
+    raw1394handle_t m_util_handle;
     int             m_port;
     std::string     m_portName;
 
-    pthread_t       m_thread;
-    Util::Mutex*    m_RHThread_lock;
-    bool            m_threadRunning;
-
     bool            m_realtime;
     int             m_base_priority;
 
     IsoHandlerManager*      m_pIsoManager;
     CycleTimerHelper*       m_pCTRHelper;
     bool                    m_have_new_ctr_read;
+    bool                    m_have_read_ctr_and_clock;
 
     bool            m_filterFCPResponse;
 
@@ -411,6 +470,12 @@ private:
     typedef std::vector< Util::Functor* > reset_handler_vec_t;
     reset_handler_vec_t m_busResetHandlers;
 
+    // ARM stuff
+    arm_tag_handler_t m_default_arm_handler;
+
+    typedef std::vector< ARMHandler * > arm_handler_vec_t;
+    arm_handler_vec_t m_armHandlers;
+
     // unprotected variants
     bool writeNoLock( fb_nodeid_t nodeId,
         fb_nodeaddr_t addr,
@@ -454,6 +519,64 @@ public:
     void show();
 private:
     DECLARE_DEBUG_MODULE;
+    
+public:
+    /**
+    * @brief Class to handle AddressRangeMappings
+    *
+    * This class is intended to help with implementing
+    * address range mapping, i.e. implementing handlers
+    * that react to reads/writes of certain addresses
+    * in 1394 memory space
+    *
+    * see the _arm_ functions in raw1394.h for more insight
+    *
+    */
+
+    class ARMHandler {
+    public:
+        ARMHandler(Ieee1394Service &parent,
+                nodeaddr_t start, size_t length,
+                unsigned int access_rights,
+                unsigned int notification_options,
+                unsigned int client_transactions
+                );
+    
+        virtual ~ARMHandler();
+    
+        virtual bool handleRead(struct raw1394_arm_request  *);
+        virtual bool handleWrite(struct raw1394_arm_request  *);
+        virtual bool handleLock(struct raw1394_arm_request  *);
+    
+        struct raw1394_arm_response *getResponse() {return &m_response;};
+    
+        nodeaddr_t getStart() {return m_start;};
+        size_t getLength() {return m_length;};
+        unsigned int getAccessRights() {return m_access_rights;};
+        unsigned int getNotificationOptions() {return m_notification_options;};
+        unsigned int getClientTransactions() {return m_client_transactions;};
+    
+        byte_t *getBuffer() {return m_buffer;};
+    
+    private:
+        Ieee1394Service &m_parent;
+        nodeaddr_t m_start;
+        size_t m_length;
+        unsigned int m_access_rights;
+        unsigned int m_notification_options;
+        unsigned int m_client_transactions;
+    protected:
+        byte_t *m_buffer;
+    
+        struct raw1394_arm_response m_response;
+        
+        void printBufferBytes( unsigned int level, size_t length, byte_t* buffer ) const;
+        void printRequest(struct raw1394_arm_request *arm_req);
+
+        DECLARE_DEBUG_MODULE_REFERENCE;
+    
+    };
+
 };
 
 #endif // FFADO_IEEE1394SERVICE_H
