@@ -22,6 +22,7 @@
  */
 
 #include "config.h"
+
 #include "AmdtpTransmitStreamProcessor.h"
 #include "AmdtpPort.h"
 #include "../StreamProcessorManager.h"
@@ -72,6 +73,7 @@ AmdtpTransmitStreamProcessor::generatePacketHeader (
     * our node ID for each packet. */
     packet->sid = m_local_node_id;
 
+    packet->eoh0 = 0;
     packet->dbs = m_dimension;
     packet->fn = 0;
     packet->qpc = 0;
@@ -306,6 +308,7 @@ AmdtpTransmitStreamProcessor::generateSilentPacketHeader (
 
     packet->sid = m_local_node_id;
 
+    packet->eoh0 = 0;
     packet->dbs = m_dimension;
     packet->fn = 0;
     packet->qpc = 0;
@@ -342,6 +345,7 @@ AmdtpTransmitStreamProcessor::generateEmptyPacketHeader (
                        (unsigned int)TICKS_TO_CYCLES(m_last_timestamp) );
     packet->sid = m_local_node_id;
 
+    packet->eoh0 = 0;
     packet->dbs = m_dimension;
     packet->fn = 0;
     packet->qpc = 0;
@@ -644,7 +648,9 @@ AmdtpTransmitStreamProcessor::encodeAudioPortsFloat(quadlet_t *data,
     for (; i < (int)m_nb_audio_ports; i++) {
         struct _MBLA_port_cache &p = m_audio_ports.at(i);
         target_event = (quadlet_t *)(data + i);
+#ifdef DEBUG
         assert(nevents + offset <= p.buffer_size );
+#endif
 
         if(likely(p.buffer && p.enabled)) {
             float *buffer = (float *)(p.buffer);
@@ -827,7 +833,9 @@ AmdtpTransmitStreamProcessor::encodeAudioPortsInt24(quadlet_t *data,
     for (; i < ((int)m_nb_audio_ports); i++) {
         struct _MBLA_port_cache &p = m_audio_ports.at(i);
         target_event = (quadlet_t *)(data + i);
+#ifdef DEBUG
         assert(nevents + offset <= p.buffer_size );
+#endif
 
         if(likely(p.buffer && p.enabled)) {
             uint32_t *buffer = (uint32_t *)(p.buffer);
@@ -912,7 +920,9 @@ AmdtpTransmitStreamProcessor::encodeAudioPortsInt24(quadlet_t *data,
     for (i = 0; i < m_nb_audio_ports; i++) {
         struct _MBLA_port_cache &p = m_audio_ports.at(i);
         target_event = (quadlet_t *)(data + i);
+#ifdef DEBUG
         assert(nevents + offset <= p.buffer_size );
+#endif
 
         if(likely(p.buffer && p.enabled)) {
             quadlet_t *buffer = (quadlet_t *)(p.buffer);
@@ -953,7 +963,9 @@ AmdtpTransmitStreamProcessor::encodeAudioPortsFloat(quadlet_t *data,
     for (i = 0; i < m_nb_audio_ports; i++) {
         struct _MBLA_port_cache &p = m_audio_ports.at(i);
         target_event = (quadlet_t *)(data + i);
+#ifdef DEBUG
         assert(nevents + offset <= p.buffer_size );
+#endif
 
         if(likely(p.buffer && p.enabled)) {
             quadlet_t *buffer = (quadlet_t *)(p.buffer);
@@ -1175,6 +1187,7 @@ next_index:
     return true;
 }
 
+//FIXME: DRY. Needs to be refactored with AmdtpReceiveStreamProcessor
 void
 AmdtpTransmitStreamProcessor::updatePortCache() {
     int idx;
@@ -1183,12 +1196,18 @@ AmdtpTransmitStreamProcessor::updatePortCache() {
         AmdtpAudioPort *port = p.port;
         p.buffer = port->getBufferAddress();
         p.enabled = !port->isDisabled();
+#ifdef DEBUG
+	p.buffer_size = port->getBufferSize();
+#endif
     }
     for (idx = 0; idx < m_nb_midi_ports; idx++) {
         struct _MIDI_port_cache& p = m_midi_ports.at(idx);
         AmdtpMidiPort *port = p.port;
         p.buffer = port->getBufferAddress();
         p.enabled = !port->isDisabled();
+#ifdef DEBUG
+	p.buffer_size = port->getBufferSize();
+#endif
     }
 }
 
