@@ -28,6 +28,7 @@
 
 #include "PortManager.h"
 
+#include "libutil/StreamStatistics.h"
 #include "libutil/TimestampedBuffer.h"
 #include "libutil/OptionContainer.h"
 
@@ -61,6 +62,9 @@ public:
     };
     ///> returns the type of the streamprocessor
     virtual enum eProcessorType getType() { return m_processor_type; };
+
+    ///> notification of a buffer size change
+    virtual bool periodSizeChanged(unsigned int new_periodsize);
 private:
     // this can only be set by the constructor
     enum eProcessorType m_processor_type;
@@ -222,6 +226,7 @@ protected: // the helper receive/transmit functions
         eCRV_XRun,
         eCRV_Again,
         eCRV_Defer,
+	eCRV_Error,
     };
     // to be implemented by the children
     // the following methods are to be implemented by receive SP subclasses
@@ -296,6 +301,7 @@ protected:
 
 //--- data buffering and accounting
 public:
+    bool setupDataBuffer();
     void getBufferHeadTimestamp ( ffado_timestamp_t *ts, signed int *fc );
     void getBufferTailTimestamp ( ffado_timestamp_t *ts, signed int *fc );
 
@@ -423,6 +429,8 @@ protected:
         float m_ticks_per_frame;
         float m_dll_bandwidth_hz;
         unsigned int m_extra_buffer_frames;
+        float m_max_fs_diff_norm;
+        signed int m_max_diff_ticks;
     private:
         bool m_in_xrun;
 
