@@ -22,7 +22,7 @@
  *
  */
 
-#include "config.h"
+
 #include "libutil/float_cast.h"
 
 #include "MotuReceiveStreamProcessor.h"
@@ -36,6 +36,8 @@
 
 #include "libutil/ByteSwap.h"
 
+#include "../../motu/motu_avdevice.h"
+
 #include <cstring>
 #include <math.h>
 #include <assert.h>
@@ -85,13 +87,25 @@ MotuReceiveStreamProcessor::MotuReceiveStreamProcessor(FFADODevice &parent, unsi
     , mb_head ( 0 )
     , mb_tail ( 0 )
 {
+    // The model needs to be easily visible since the interpretation of the
+    // receive stream is dependent on the model in a slight but important
+    // way.
+    Motu::MotuDevice *dev = static_cast<Motu::MotuDevice *>(&parent);
+    m_motu_model = dev->m_motu_model;
     memset(&m_devctrls, 0, sizeof(m_devctrls));
 }
 
 unsigned int
 MotuReceiveStreamProcessor::getMaxPacketSize() {
     int framerate = m_Parent.getDeviceManager().getStreamProcessorManager().getNominalRate();
-    return framerate<=48000?616:(framerate<=96000?1032:1160);
+    // What's returned here is the maximum packet size seen at the current
+    // frame rate.  This depends both on the device and its configuration,
+    // and is futher complicated by the IN/OUT channel asymmetry in some
+    // devices.  To avoid most of these complications, just return the
+    // largest packet sizes seen by any supported MOTU device.  Presently
+    // all these sizes come from the receive side of the 828mk3.  
+    // Previously the figures came from the Traveler (I think).
+    return framerate<=48000?904:(framerate<=96000?1416:1672);
 }
 
 unsigned int
@@ -122,6 +136,10 @@ MotuReceiveStreamProcessor::processPacketHeader(unsigned char *data, unsigned in
                                                 unsigned char tag, unsigned char sy,
                                                 uint32_t pkt_ctr)
 {
+    static int len_shown = 0;
+    static int ts_print_cx = 0;
+static int pktcx = 0;
+
     if (length > 8) {
         // The iso data blocks from the MOTUs comprise a CIP-like
         // header followed by a number of events (8 for 1x rates, 16
@@ -137,7 +155,13 @@ MotuReceiveStreamProcessor::processPacketHeader(unsigned char *data, unsigned in
         // standards.  Do this check early on because for invalid packets
         // dbs may not be what we expect, potentially causing issues later
         // on.
-        if (tag!=1 || fdf_size!=32 || dbs==0) {
+        //
+        // Mark 3 devices are a little different.  Previous generations set
+        // the fdf_size to 32.  Mark 3 devices appear to set fdf_size to 0. 
+        // Allow for this at present.  However, if too many variations
+        // appear we might have to omit this test in the interests of
+        // brevity.
+        if (tag!=1 || (fdf_size!=32 && fdf_size!=0) || dbs==0) {
             return eCRV_Invalid;
         }
 
@@ -150,6 +174,37 @@ MotuReceiveStreamProcessor::processPacketHeader(unsigned char *data, unsigned in
         uint32_t last_sph = CondSwapFromBus32(*(quadlet_t *)(data+8+(n_events-1)*m_event_size));
         m_last_timestamp = sphRecvToFullTicks(last_sph, m_Parent.get1394Service().getCycleTimer());
 
+        // To assist in debugging the packet format from new devices, periodically
+        // dump a packet when debug is active.  Originally written to debug the
+        // Ultralite mk3 it has been retained since it could prove useful in
+        // future.
+        if ((!len_shown || pktcx==0) && getDebugLevel()>0 ) {
+            unsigned int i;
+            fprintf(stderr, "Packet from MOTU: length=%d, eventsize=%d, n_events=%d\n", length, m_event_size, n_events);
+            for (i=0; i<length; i++) {
+              if ((i&0x000f) == 0)
+                fprintf(stderr, "%08x  ", i);
+              fprintf(stderr, "%02x ", data[i]);
+              if ((i&0x000f) == 7)
+                fprintf(stderr, "- ");
+              if ((i&0x000f) == 0xf)
+                fprintf(stderr, "\n");
+            }
+            fprintf(stderr, "\n");
+        }
+        // Print packet contents once per second
+        if (++pktcx == 8000)
+            pktcx=0;
+
+        // For now suppress packet printing after the first packet
+        len_shown = 1;
+
+        // If debug is active print the first 20 packet timestamps.
+        if (ts_print_cx<20 && getDebugLevel()>0 ) {
+          debugOutput(DEBUG_LEVEL_VERBOSE,"last ts=0x%08x\n", last_sph);
+          ts_print_cx++;
+        }
+
         return eCRV_OK;
     } else {
         return eCRV_Invalid;
@@ -204,8 +259,12 @@ bool MotuReceiveStreamProcessor::processReadBlock(char *data,
 {
     bool no_problem=true;
 
-    /* Scan incoming block for device control events */
-    decodeMotuCtrlEvents(data, nevents);
+    /* Scan incoming block for device control events.  These do not seem to
+     * be present on the 828Mk1 (or at the very least are in a different
+     * location).
+     */
+    if (m_motu_model != Motu::MOTU_MODEL_828MkI)
+        decodeMotuCtrlEvents(data, nevents);
 
     for ( PortVectorIterator it = m_Ports.begin();
           it != m_Ports.end();
@@ -496,6 +555,10 @@ MotuReceiveStreamProcessor::decodeMotuCtrlEvents(
                     // MIDI is dealt with elsewhere, so just pass it over
                     break;
                 default:
+                    // Uncomment to debug unknown keys
+                    // debugOutput(DEBUG_LEVEL_VERBOSE, "Unknown MOTU key %d, value %d\n",
+                    //   control_key, (signed int)(*arg));
+
                     // Ignore any unknown keys or those we don't care about, at
                     // least for now.
                     break;
