@@ -185,12 +185,19 @@ public:
 
     virtual void show();
 
+    bool hasNames() const { return true; }
+    bool canConnect() const { return false; }
+
     virtual std::string getRowName(const int row);
     virtual std::string getColName(const int col);
     virtual int canWrite( const int, const int ) { return true; }
     virtual int getRowCount();
     virtual int getColCount();
 
+    // full map updates are unsupported
+    virtual bool getCoefficientMap(int &) {return false;};
+    virtual bool storeCoefficientMap(int &) {return false;};
+
 protected:
      struct sSignalInfo {
          std::string name;
