@@ -38,6 +38,7 @@
 #include "debugmodule/debugmodule.h"
 
 #include "devicemanager.h"
+#include "version.h"
 
 #include <iostream>
 #include <string>
@@ -104,7 +105,7 @@ main( int argc, char** argv )
     printf("This program comes with ABSOLUTELY NO WARRANTY.\n");
     printf("-----------------------------------------------\n\n");
 
-    memset(args, 0, sizeof(args));
+    memset(args, 0, sizeof(*args));
 
     args->guid = 0xFFFFFFFFFFFFFFFFLL;
     argp_parse (argp, argc, argv, 0, 0, args);
